/* ControlManager.m - Menu suppor class to 
   the Control menu functions for the 
   Macintosh OS X SDL port of Atari800
   Mark Grebe <atarimac@cox.net>
   
   Based on the Preferences pane of the
   TextEdit application.

*/
#import "ControlManager.h"
#import "Preferences.h"
#import "MediaManager.h"
#import "AboutBox.h"
#import "KeyMapper.h"
#import "SDL.h"
#import <stdarg.h>


/* Definition of Mac native keycodes for characters used as menu shortcuts the bring up a windo. */
#define QZ_F8			0x64

#define FUNCTION_KEY_PRESS_DURATION 5 /* Duration of holding the function key down in machine cycles */

extern void PauseAudio(int pause);
extern void monitorEnter(void);
extern int monitorCmd(char *input);
extern int CalcAtariType(int machineType, int ramSize);

extern int pauseEmulator;
extern int requestPauseEmulator;
extern int requestColdReset;
extern int requestWarmReset;
extern int requestSaveState;
extern int requestLoadState;
extern int requestLimitChange;
extern int requestDisableBasicChange;
extern int requestKeyjoyEnableChange;
extern int requestMachineTypeChange;
extern int requestQuit;
extern int speed_limit;
extern char saveFilename[FILENAME_MAX];
extern char loadFilename[FILENAME_MAX];
extern char atari_state_dir[FILENAME_MAX];
extern int assemblerMode;
extern int functionKeysWindowOpen;
/* Function key window interface variables */
extern int breakFunctionPressed;
extern int startFunctionPressed;
extern int selectFunctionPressed;
extern int optionFunctionPressed;
extern int FULLSCREEN;


/* Functions which provide an interface for C code to call this object's shared Instance functions */
void SetControlManagerLimit(int limit) {
    [[ControlManager sharedInstance] setLimitMenu:(limit)];
    }
    
void SetControlManagerDisableBasic(int disableBasic) {
    [[ControlManager sharedInstance] setDisableBasicMenu:(disableBasic)];
    }
    
void SetControlManagerKeyjoyEnabe(int keyjoyEnable) {
    [[ControlManager sharedInstance] setKeyjoyEnableMenu:(keyjoyEnable)];
    }
    
void SetControlManagerMachineType(int machineType, int ramSize) {
    [[ControlManager sharedInstance] setMachineTypeMenu:(machineType):(ramSize)];
    }
    
int ControlManagerFatalError() {
    return([[ControlManager sharedInstance] fatalError]);
    }

void ControlManagerSaveState() {
    [[ControlManager sharedInstance] saveState:nil];
}

void ControlManagerLoadState() {
    [[ControlManager sharedInstance] loadState:nil];
}

void ControlManagerPauseEmulator() {
    [[ControlManager sharedInstance] pause:nil];
}

void ControlManagerHideApp() {
    [NSApp hide:[ControlManager sharedInstance]];
    [[KeyMapper sharedInstance] releaseCmdKeys:@"h"];
}

void ControlManagerAboutApp() {
    [NSApp orderFrontStandardAboutPanel:[ControlManager sharedInstance]];
    [[KeyMapper sharedInstance] releaseCmdKeys:@"a"];
}

void ControlManagerShowHelp() {
    [NSApp showHelp:[ControlManager sharedInstance]];
    [[KeyMapper sharedInstance] releaseCmdKeys:@"?"];
}

void ControlManagerMiniturize() {
    [[NSApp keyWindow] performMiniaturize:[ControlManager sharedInstance]];
    [[KeyMapper sharedInstance] releaseCmdKeys:@"m"];
}

void ControlManagerMessagePrint(char *string)
{
[[ControlManager sharedInstance] messagePrint:string];
}

void ControlManagerMonitorPrintf(const char *format,...)
{
	static char string[512];
	va_list arguments;             
    va_start(arguments, format);  
    
	vsprintf(string, format, arguments);
    [[ControlManager sharedInstance] monitorPrint:string];
}

void ControlManagerMonitorPuts(const char *string)
{
    [[ControlManager sharedInstance] monitorPrint:string];
}

int ControlManagerMonitorRun()
{
    return([[ControlManager sharedInstance] monitorRun]);
}

void ControlManagerFunctionKeysWindowShow(void) 
{
	[[ControlManager sharedInstance] functionKeyWindowShow:nil];
}

char *ControlManagerGetHistoryString(int direction)
{
	NSString *historyString;
	static char buffer[256];
	
	historyString = [[ControlManager sharedInstance] getHistoryString:direction];
	if (historyString == nil)
		return(NULL);
	else {
		[historyString getCString:buffer maxLength:255];
		return(buffer);
		}
}

void ControlManagerAddToHistory(char *string)
{
	NSString *historyString;
	
	historyString = [NSString stringWithCString:string];
	[[ControlManager sharedInstance] addToHistory:historyString];
	[historyString release];
}

@implementation ControlManager
static ControlManager *sharedInstance = nil;
#define MAX_MONITOR_OUTPUT 4096
static char monitorOutput[MAX_MONITOR_OUTPUT];
static int monitorCharCount = 0;

+ (ControlManager *)sharedInstance {
    return sharedInstance ? sharedInstance : [[self alloc] init];
}

- (id)init {
    NSDictionary *attribs;
	int i;

    if (sharedInstance) {
	[self dealloc];
    } else {
        [super init];
        sharedInstance = self;
        if (!errorTextField) {
			if ([[Preferences sharedInstance] getBrushed]) {
				if (![NSBundle loadNibNamed:@"ControlManagerBrushed" owner:self])  {
					NSLog(@"Failed to load ControlManager.nib");
					NSBeep();
					return nil;
				}
			}
			else {
				if (![NSBundle loadNibNamed:@"ControlManager" owner:self])  {
					NSLog(@"Failed to load ControlManager.nib");
					NSBeep();
					return nil;
                }
			}
            }
	[[errorTextField window] setExcludedFromWindowsMenu:YES];
	[[errorTextField window] setMenu:nil];
	[[monitorInputField window] setExcludedFromWindowsMenu:YES];
	[[monitorInputField window] setMenu:nil];
	[[messageOutputView window] setExcludedFromWindowsMenu:NO];

	attribs = [[NSDictionary alloc] initWithObjectsAndKeys:
                   [NSFont fontWithName:@"Monaco" size:10.0], NSFontAttributeName,
                   nil]; 
    [monitorOutputView setTypingAttributes:attribs];
    [messageOutputView setTypingAttributes:attribs];
    [monitorOutputView setString:@"Atari800MacX Monitor\nType '?' for help, 'CONT' to exit\n"];
    [attribs release];
	
	// Init Monitor Command History
    historyIndex = 0;
    historyLine = 0;
    historySize = 0;
    for (i = 0; i < kHistorySize; i++)
       history[i] = nil;
    }
    return sharedInstance;
}

/*------------------------------------------------------------------------------
*  setLimitMenu - This method is used to set the menu check state for the 
*     limit emulator speed menu item.
*-----------------------------------------------------------------------------*/
- (void)setLimitMenu:(int)limit
{
    if (limit)
        [limitItem setState:NSOnState];
    else
        [limitItem setState:NSOffState];
		
	[[MediaManager sharedInstance] setLimitButton:limit];
}

/*------------------------------------------------------------------------------
*  setDisableBasicMenu - This method is used to set the menu check state for the 
*     Disable Basic menu item.
*-----------------------------------------------------------------------------*/
- (void)setDisableBasicMenu:(int)disableBasic
{
    if (disableBasic)
        [disableBasicItem setState:NSOnState];
    else
        [disableBasicItem setState:NSOffState];
		
	[[MediaManager sharedInstance] setDisableBasicButton:disableBasic];
}

/*------------------------------------------------------------------------------
*  setKeyjoyEnableMenu - This method is used to set the menu check state for the 
*     Enable Keyboard Joystcks menu item.
*-----------------------------------------------------------------------------*/
- (void)setKeyjoyEnableMenu:(int)keyjoyEnable
{
    if (keyjoyEnable)
        [keyjoyEnableItem setState:NSOnState];
    else
        [keyjoyEnableItem setState:NSOffState];
}

/*------------------------------------------------------------------------------
*  setMachineTypeMenu - This method is used to set the menu check state for the 
*     Machine Type menu items.
*-----------------------------------------------------------------------------*/
- (void)setMachineTypeMenu:(int)machineType:(int)ramSize
{
	int i, type;
    
	type = CalcAtariType(machineType, ramSize);
	
	for (i=0;i<14;i++) {
		if (i==type)
			[[machineTypeMenu itemAtIndex:i] setState:NSOnState];
		else
			[[machineTypeMenu itemAtIndex:i] setState:NSOffState];
		}
}

- (IBAction)changeMachineType:(id)sender 
{
	requestMachineTypeChange = [sender tag] + 1;
}


/*------------------------------------------------------------------------------
*  browseFileInDirectory - This allows the user to chose a file to read in from
*     the specified directory.
*-----------------------------------------------------------------------------*/
- (NSString *) browseFileInDirectory:(NSString *)directory {
    NSOpenPanel *openPanel = nil;
    
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseDirectories:NO];
    [openPanel setCanChooseFiles:YES];
    
    if ([openPanel runModalForDirectory:directory file:nil types:nil] == NSOKButton)
        return([[openPanel filenames] objectAtIndex:0]);
    else
        return nil;
    }

/*------------------------------------------------------------------------------
*  saveFileInDirectory - This allows the user to chose a filename to save in from
*     the specified directory.
*-----------------------------------------------------------------------------*/
- (NSString *) saveFileInDirectory:(NSString *)directory:(NSString *)type {
    NSSavePanel *savePanel = nil;
    
    savePanel = [NSSavePanel savePanel];
    
    [savePanel setRequiredFileType:type];
    
    if ([savePanel runModalForDirectory:directory file:nil] == NSOKButton)
        return([savePanel filename]);
    else
        return nil;
    }

/*------------------------------------------------------------------------------
*  coldReset - This method handles the cold reset menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)coldReset:(id)sender
{
    requestColdReset = 1;
}

/*------------------------------------------------------------------------------
*  limit - This method handles the limit emulator speed menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)limit:(id)sender
{
    requestLimitChange = 1;
}

/*------------------------------------------------------------------------------
*  disableBasic - This method handles the disable basic menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)disableBasic:(id)sender
{
    requestDisableBasicChange = 1;
}

/*------------------------------------------------------------------------------
*  keyjoyEnable - This method handles the Enable Keyboard Joysticks menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)keyjoyEnable:(id)sender
{
    requestKeyjoyEnableChange = 1;
}

/*------------------------------------------------------------------------------
*  loadState - This method handles the load state file menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)loadState:(id)sender
{
    NSString *filename;
    
    PauseAudio(1);
    filename = [self browseFileInDirectory:[NSString stringWithCString:atari_state_dir]];
    if (filename != nil) {
        [filename getCString:loadFilename];
        requestLoadState = 1;
    }
	[[MediaManager sharedInstance] updateInfo];
    [[KeyMapper sharedInstance] releaseCmdKeys:@"l"];
    PauseAudio(0);
}

/*------------------------------------------------------------------------------
*  loadStateFile - This method handles the load state file menu selection.
*-----------------------------------------------------------------------------*/
- (void)loadStateFile:(NSString *)filename
{
    if (filename != nil) {
        [filename getCString:loadFilename];
        requestLoadState = 1;
    }
}

/*------------------------------------------------------------------------------
*  pause - This method handles the pause emulator menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)pause:(id)sender
{
    if (1 - pauseEmulator)
        [pauseItem setState:NSOnState];
    else
        [pauseItem setState:NSOffState];
    requestPauseEmulator = 1;
}

/*------------------------------------------------------------------------------
*  saveState - This method handles the save state file menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)saveState:(id)sender
{
    NSString *filename;
    
    PauseAudio(1);
    filename = [self saveFileInDirectory:[NSString stringWithCString:atari_state_dir]:@"a8s"];
    if (filename != nil) {
        [filename getCString:saveFilename];
        requestSaveState = 1;
    }
    [[KeyMapper sharedInstance] releaseCmdKeys:@"s"];
    PauseAudio(0);
}

/*------------------------------------------------------------------------------
*  warmReset - This method handles the warm reset menu selection.
*-----------------------------------------------------------------------------*/
- (IBAction)warmReset:(id)sender
{
    requestWarmReset = 1;
}

/*------------------------------------------------------------------------------
*  fatalError - This method displays the fatal error dialogue box.
*-----------------------------------------------------------------------------*/
- (int)fatalError
{
    return([NSApp runModalForWindow:[errorTextField window]]);
}

/*------------------------------------------------------------------------------
*  coldStartSelected - This method handles the warm start button from the fatal 
*     error dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)coldStartSelected:(id)sender
{
    [NSApp stopModalWithCode:2];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  warmStartSelected - This method handles the warm start button from the fatal 
*     error dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)warmStartSelected:(id)sender
{
    [NSApp stopModalWithCode:1];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  monitorSelected - This method handles the monitor button from the fatal 
*     error dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)monitorSelected:(id)sender
{
    [NSApp stopModalWithCode:3];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  ejectCartridgeSelected - This method handles the eject cartridge button from 
*     the fatal error dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)ejectCartridgeSelected:(id)sender
{
    [NSApp stopModalWithCode:4];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  ejectDiskSelected - This method handles the eject disk 1 button from 
*     the fatal error dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)ejectDiskSelected:(id)sender
{
    [NSApp stopModalWithCode:5];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  quitSelected - This method handles the quit button from the fatal error
*     dialogue.
*-----------------------------------------------------------------------------*/
- (IBAction)quitSelected:(id)sender
{
    [NSApp stopModalWithCode:0];
    [[sender window] close];
}

/*------------------------------------------------------------------------------
*  releaseKey - This method fixes an issue when modal windows are used with
*     the Mac OSX version of the SDL library.
*     As the SDL normally captures all keystrokes, but we need to type in some 
*     Mac windows, all of the control menu windows run in modal mode.  However, 
*     when this happens, the release of function key which started the process
*     is not sent to SDL.  We have to manually cause these events to happen 
*     to keep the SDL library in a sane state, otherwise only everyother shortcut
*     keypress will work.
*-----------------------------------------------------------------------------*/
- (void) releaseKey:(int)keyCode
{
    NSEvent *event1;
    NSPoint point;
    
    event1 = [NSEvent keyEventWithType:NSKeyUp location:point modifierFlags:0
                    timestamp:nil windowNumber:0 context:nil characters:@" "
                    charactersIgnoringModifiers:@" " isARepeat:NO keyCode:keyCode];
    [NSApp postEvent:event1 atStart:NO];
}

/*------------------------------------------------------------------------------
*  monitorExecute - This method handles the execute button (or return key) from
*     the monitor window.
*-----------------------------------------------------------------------------*/
- (IBAction)monitorExecute:(id)sender
{
    char input[256];
    int retValue = 0;
	NSString *line;
   
    line = [monitorInputField stringValue];
	[self addToHistory:line];
    [line getCString:input];
    
    monitorCharCount = 0;
    [self monitorPrint:input];
    [self monitorPrint:"\n"];
    retValue = monitorCmd(input);
    if (!assemblerMode && retValue!=-1)
        [self monitorPrint:"> "];

    [NSApp stopModalWithCode:retValue];
}

/*------------------------------------------------------------------------------
*  messageWindowShow - This method makes the emulator message window visable
*-----------------------------------------------------------------------------*/
- (void)messageWindowShow:(id)sender
{
	static int firstTime = 1;

	if (firstTime) {
		[[messageOutputView window] setFrameOrigin:[[Preferences sharedInstance] messagesOrigin]];
		firstTime = 0;
		}
	
    [[messageOutputView window] makeKeyAndOrderFront:self];
	[[messageOutputView window] setTitle:@"Emulator Messages"];
	
}

/*------------------------------------------------------------------------------
*  functionKeyWindowShow - This method makes the function keys window visable
*-----------------------------------------------------------------------------*/
- (IBAction)functionKeyWindowShow:(id)sender
{
	static int firstTime = 1;

	if (firstTime && !FULLSCREEN) {
		[[startButton window] setFrameOrigin:[[Preferences sharedInstance] functionKeysOrigin]];
		firstTime = 0;
		}
		
	functionKeysWindowOpen = TRUE;	
    [[startButton window] makeKeyAndOrderFront:self];
}

/*------------------------------------------------------------------------------
*  messagesOriginSave - This method saves the position of the messages
*    window
*-----------------------------------------------------------------------------*/
- (NSPoint)messagesOriginSave
{
	NSRect frame;
	
	frame = [[messageOutputView window] frame];
	return(frame.origin);
}

/*------------------------------------------------------------------------------
*  funcitonKeysOriginSave - This method saves the position of the function keys
*    window
*-----------------------------------------------------------------------------*/
- (NSPoint)functionKeysOriginSave;
{
	NSRect frame;
	
	frame = [[startButton window] frame];
	return(frame.origin);
}

/*------------------------------------------------------------------------------
*  monitorOriginSave - This method saves the position of the monitor
*    window
*-----------------------------------------------------------------------------*/
- (NSPoint)monitorOriginSave
{
	NSRect frame;
	
	frame = [[monitorOutputView window] frame];
	return(frame.origin);
}


/*------------------------------------------------------------------------------
*  messagePrint - This method handles the printing of information to the
*     message window.  It replaces printf.
*-----------------------------------------------------------------------------*/
- (void)messagePrint:(char *)printString
{
    NSRange theEnd;
    NSString *stringObj;

    theEnd=NSMakeRange([[messageOutputView string] length],0);
    stringObj = [[NSString alloc] initWithCString:printString];
    [messageOutputView replaceCharactersInRange:theEnd withString:stringObj]; // append new string to the end
    theEnd.location += strlen(printString); // the end has moved
	[stringObj autorelease];
    [messageOutputView scrollRangeToVisible:theEnd];
}

/*------------------------------------------------------------------------------
*  monitorPrint - This method handles the printing of information from the
*     monitor routines.  It replaces printf.
*-----------------------------------------------------------------------------*/
- (void)monitorPrint:(char *)string
{
    strncpy(&monitorOutput[monitorCharCount], string, MAX_MONITOR_OUTPUT - monitorCharCount);
    monitorCharCount += strlen(string);
}

/*------------------------------------------------------------------------------
*  monitorMenuRun - This method runs the monitor from the menu, and calls the
*     normal key based monitor run.
*-----------------------------------------------------------------------------*/
- (IBAction)monitorMenuRun:(id)sender
{
    int retValue;
    
    retValue = [self monitorRun];
    if (retValue == 0) 
        requestQuit = 1;
}

/*------------------------------------------------------------------------------
*  monitorRun - This method runs the monitor, and contains the main loop, one
*     loop for each command entered.
*-----------------------------------------------------------------------------*/
-(int)monitorRun
{
    int retValue = 0;
    NSRange theEnd;
    NSString *stringObj;
	static int firstTime = 1;

	if (firstTime) {
		[[monitorOutputView window] setFrameOrigin:[[Preferences sharedInstance] monitorOrigin]];
		firstTime = 0;
		}

    PauseAudio(1);
            
    monitorEnter();
    [self monitorPrint:"> "];
    theEnd=NSMakeRange([[monitorOutputView string] length],0);
    stringObj = [[NSString alloc] initWithCString:monitorOutput];
    [monitorOutputView replaceCharactersInRange:theEnd withString:stringObj]; // append new string to the end
    theEnd.location += monitorCharCount; // the end has moved
    [monitorOutputView scrollRangeToVisible:theEnd];
    
    while (retValue == 0) {
        retValue = [NSApp runModalForWindow:[monitorInputField window]];
        theEnd=NSMakeRange([[monitorOutputView string] length],0);
		[stringObj release];
        stringObj = [[NSString alloc] initWithCString:monitorOutput];
        [monitorOutputView replaceCharactersInRange:theEnd withString:stringObj]; // append new string to the end
        theEnd.location += monitorCharCount; // the end has moved
        [monitorOutputView scrollRangeToVisible:theEnd];
		[monitorInputField setStringValue:@""];
		[[monitorInputField window] makeFirstResponder:monitorInputField];
		}
    
    monitorCharCount = 0;
    [[monitorInputField window] close];
    [self releaseKey:QZ_F8];

    PauseAudio(0);
    
    if (retValue == -1)
        return(1);
    else
        return(0);
}

/*------------------------------------------------------------------------------
*  monitorUpArrow - Handle the user pressing the up arrow in the monitor window
*-----------------------------------------------------------------------------*/
- (void)monitorUpArrow
{
	[self historyScroll:1];
}

/*------------------------------------------------------------------------------
*  monitorDownArrow - Handle the user pressing the down arrow in the monitor 
*      window
*-----------------------------------------------------------------------------*/
- (void)monitorDownArrow
{
	[self historyScroll:-1];
}

/*------------------------------------------------------------------------------
*  addToHistory - Add the current command to the history list
*-----------------------------------------------------------------------------*/
- (void)addToHistory:(NSString *)str
{
	[str retain];
	[history[historyIndex] release];
	history[historyIndex] = str;
    historyIndex = (historyIndex + 1) % kHistorySize;
	historyLine = 0;
	
	if (historySize < kHistorySize)
		historySize++;
}

/*------------------------------------------------------------------------------
*  historyScroll - Scroll through the commands in the history in the specified
*     direction.
*-----------------------------------------------------------------------------*/
- (void)historyScroll:(int)direction
{
	NSString *historyString;
	
	historyString = [self getHistoryString:direction];
	
	if (historyString == nil)
		return;
		
	[monitorInputField setStringValue:historyString];
}

/*------------------------------------------------------------------------------
*  getHistoryString - Get the next histroy string in the specified direction
*    from the current one.
*-----------------------------------------------------------------------------*/
- (NSString *)getHistoryString:(int)direction
{
	int line;
	int index;
	
	if (historySize == 0)
		return nil;

	// Advance to the next line in the history
	line = historyLine + direction;
	if ((direction < 0 && line <= 0) || (direction > 0 && line > historySize))
		return nil;
	historyLine = line;
	
	if (historyLine > 0)
		index = (historyIndex - historyLine + historySize) % historySize;
	else
		index = historyIndex;
	
	return(history[index]);
}

/*------------------------------------------------------------------------------
*  showAboutBox - Show the about box
*-----------------------------------------------------------------------------*/
- (IBAction)showAboutBox:(id)sender
{
    [[AboutBox sharedInstance] showPanel:sender];
}

/*------------------------------------------------------------------------------
*  showDonation - Show the donation page in the web browser.
*-----------------------------------------------------------------------------*/
- (IBAction)showDonation:(id)sender;
{
	[[NSWorkspace  sharedWorkspace] openURL:
		[NSURL URLWithString:@"http://order.kagi.com/?6FBTU&lang=en"]];
}

/*------------------------------------------------------------------------------
*  startPressed - Handle the user pressing the start button in the function
*     keys window.  We want to hold it down for Duration frames.
*-----------------------------------------------------------------------------*/
- (IBAction)startPressed:(id)sender
{
	startFunctionPressed = FUNCTION_KEY_PRESS_DURATION;
}

/*------------------------------------------------------------------------------
*  breakPressed - Handle the user pressing the start button in the function
*     keys window.  We want to hold it down for Duration frames.
*-----------------------------------------------------------------------------*/
- (IBAction)breakPressed:(id)sender
{
	breakFunctionPressed = FUNCTION_KEY_PRESS_DURATION;
}

/*------------------------------------------------------------------------------
*  optionPressed - Handle the user pressing the option button in the function
*     keys window.  We want to hold it down for Duration frames.
*-----------------------------------------------------------------------------*/
- (IBAction)optionPressed:(id)sender
{
	optionFunctionPressed = FUNCTION_KEY_PRESS_DURATION;
}

/*------------------------------------------------------------------------------
*  selectPressed - Handle the user pressing the select button in the function
*     keys window.  We want to hold it down for Duration frames.
*-----------------------------------------------------------------------------*/
- (IBAction)selectPressed:(id)sender
{
	selectFunctionPressed = FUNCTION_KEY_PRESS_DURATION;
}



@end
